/* Seamless3d Server

Copyright (C) 2002-2009 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#pragma warning (disable : 4267 4244 4305 4100 4355 4996)

#define STANDARDBUFLEN 20

void searchTillText(char*& p);
float atof2(const char* str);
void searchTillNextWord(char*& p);
template <class T>class Ptr;

class String;
class VrmlMatrix;
class Vec3f {
public:

	float x;
	float y;
	float z;
	Vec3f(){
		x=0;
		y=0;
		z=0;
	}

	Vec3f(float nx,float ny,float nz){
		x=nx;
		y=ny;
		z=nz;
	}
/*
	Vec3f(char* str){
		char* p=str;
		searchTillText(p);
		x=atof2(p);
		searchTillNextWord(p);
		y=atof2(p);
		searchTillNextWord(p);
		z=atof2(p);
	}
*/
//    Vec3f&	operator =(SF<Vec3f,	float,	SFVEC3F>& par);
//    Vec3f&	operator -=(SF<Vec3f,	float,	SFVEC3F>& par);
//    Vec3f&	operator +=(SF<Vec3f,	float,	SFVEC3F>& par);
//    Vec3f&	operator /=(SF<Vec3f,	float,	SFVEC3F>& par);
//    Vec3f&	operator *=(SF<Vec3f,	float,	SFVEC3F>& par);

    Vec3f	operator /(const float& v)const{
		return (Vec3f(x/v, y/v, z/v));
	}
    Vec3f	operator +(const Vec3f& v)const{
		return (Vec3f(x+v.x, y+v.y, z+v.z));
	}
    Vec3f	operator -(const Vec3f& v)const{
		return (Vec3f(x-v.x, y-v.y, z-v.z));
	}
    Vec3f	operator -()const{
		return (Vec3f(-x, -y, -z));
	}

    Vec3f	operator *(const Vec3f& par)const{
		return (Vec3f(x*par.x, y*par.y, z*par.z));
	}
    Vec3f	operator *(float par)const{
		return (Vec3f(x*par, y*par, z*par));
	}
//	friend Vec3f operator *(const float& par0, const Vec3f& par1){
//		return (Vec3f(par0*par1.x, par0*par1.y, par0*par1.z ));
//	}

    Vec3f	operator /(const Vec3f& v)const{
		return (Vec3f(x/v.x, y/v.y, z/v.z));
	}

    Vec3f&	operator -=(const Vec3f& par1){
		x-=par1.x;
		y-=par1.y;
		z-=par1.z;
		return *this;
	}
    Vec3f&	operator +=(const Vec3f& par1){
		x+=par1.x;
		y+=par1.y;
		z+=par1.z;
		return *this;
	}
    Vec3f&	operator /=(const Vec3f& par1){
		x/=par1.x;
		y/=par1.y;
		z/=par1.z;
		return *this;
	}
    Vec3f&	operator *=(const Vec3f& par1){
		x*=par1.x;
		y*=par1.y;
		z*=par1.z;
		return *this;
	}
    Vec3f&	operator /=(const float& par1){
		x/=par1;
		y/=par1;
		z/=par1;
		return *this;
	}
    Vec3f&	operator *=(const float& par1){
		x*=par1;
		y*=par1;
		z*=par1;
		return *this;
	}
    friend bool	operator !=(const Vec3f& par1, const Vec3f& par2){
		if(
			par1.x==par2.x&&
			par1.y==par2.y&&
			par1.z==par2.z
			)
			return false;
		else return true;
	}
    friend bool	operator ==(const Vec3f& par1, const Vec3f& par2){
		if(
			par1.x==par2.x&&
			par1.y==par2.y&&
			par1.z==par2.z
			)
			return true;
		else return false;
	}
	float dot(const Vec3f& v) const{
		return (x * v.x +	y * v.y +  z * v.z);
	}
	Vec3f cross ( const Vec3f& v)const{
	   return Vec3f(y * v.z - z * v.y,
					z * v.x - x * v.z,
					x * v.y - y * v.x);
	}
	String toString();
	void toString(String& str);

	Vec3f operator *=(const VrmlMatrix matrix);
	Vec3f normalize()const;
	float length()const;
	void getField(const String& str, int& i);
};
class Node{
public:
	unsigned refCount;
	void incRefCount(){
		if(this!=NULL){ //is this line really needed?
		   ++refCount;
		}
	}
	unsigned decRefCount(){
		--refCount;
		if(refCount<0) throw this; //is this line really needed?
		return refCount;
	}
	virtual ~Node(){};
};

class Quaternion{
public:
   	double x, y, z, w;
	operator double *() { return &x; }
	Quaternion(){
	}
	Quaternion(double i_x, double i_y, double i_z, double i_w):
		x(i_x),
		y(i_y),
		z(i_z),
		w(i_w)
	{}
};
class Rotation {
public:
	float x;
	float y;
	float z;
	float angle;
	Rotation (){
		x=0;
		y=1;
		z=0;
		angle=0;
	}
	Rotation (const float i_x, const float i_y, const float i_z, const float i_angle){
		x=i_x;
		y=i_y;
		z=i_z;
		angle=i_angle;
	}
	Rotation (const Vec3f vec, const float i_angle){
		x=vec.x;
		y=vec.y;
		z=vec.z;
		angle=i_angle;
	}
	Rotation (Vec3f fromVector, Vec3f toVector);

	Rotation(char* str){
		char* p=str;
		searchTillText(p);
		x=atof2(p);
		searchTillNextWord(p);
		y=atof2(p);
		searchTillNextWord(p);
		z=atof2(p);
		searchTillNextWord(p);
		angle=atof2(p);
	}
    friend bool	operator !=(const Rotation &par1, const Rotation &par2){
		if(
			par1.x==par2.x&&
			par1.y==par2.y&&
			par1.z==par2.z&&
			par1.angle==par2.angle
			)
			return false;
		else return true;
	}
    friend bool	operator ==(const Rotation &par1, const Rotation &par2){
		if(
			par1.x==par2.x&&
			par1.y==par2.y&&
			par1.z==par2.z&&
			par1.angle==par2.angle
			)
			return true;
		else return false;
	}
    Rotation	operator *=(const Rotation& par1);
    Rotation	operator *(const Rotation& par1)const;
	Vec3f operator *(const Vec3f& vec)const;

	String toString()const;
	Vec3f getAxis()const;
	const Vec3f& setAxis(const Vec3f& par){
		x=par.x;
		y=par.y;
		z=par.z;
		return par;
	};
	Rotation operator -()const;
	void getField(const String& str, int& i);
	Rotation slerp(Rotation dest, float amount)const;
};
//String
//Copyright  2002,2009 written by Graham John Perrett thyme@seamless3d.com www.seamless3d.com
#include <stdio.h>

extern char floatFormat[];
extern char floatFormat2[];

#define defaultBufLen 0x200
void removeZeros(char* str);
class String {
public:
	class Data {
	public:
		unsigned refCount;
		unsigned bufLen;
		unsigned rawLen;
		char* rawChar;
	};
	friend class Data;
class Length{
private:
	Data*& pointer;
public:
	Length(	Data*& i_pointer):
	  pointer(i_pointer)
	{
	}
	operator int ()const{
		return pointer->rawLen;
	}
};

public:
	Data* createData(int i_bufLength=defaultBufLen, int length=0)const{
		Data* data=(Data*)new char[i_bufLength+sizeof(Data)+1];
		data->rawChar=(char*)data+sizeof(Data);
		data->refCount=1;	//must always be created with a pointer to it
		data->bufLen=i_bufLength;
		data->rawLen=length;
		return data;
	}
	String( Data* data):
		length(pointer)
	{
		pointer=data;
	}

	String(int bufSize, int length):
		length(pointer)
	{
		if(bufSize < length)throw this;
		pointer = createData(bufSize, length);
	}

	String(const String& par1):
		length(pointer)
	{
		pointer=par1.pointer;
		pointer->refCount++;
	}
	String(const char* par1=""):
		length(pointer)
	{
		unsigned lEn=strlen(par1);
		pointer=createData(lEn+defaultBufLen);
		pointer->rawLen=lEn;
		for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=par1[i];
	}
	String(char par1):
		length(pointer)
	{
		pointer=createData(1+defaultBufLen);
		pointer->rawLen=1;
		pointer->rawChar[0]=par1;
		pointer->rawChar[1]=0;
	}
	String(int par1):
		length(pointer)
	{
		char tmp[50];
		sprintf(tmp,"%i",par1);
		unsigned lEn=strlen(tmp);
		pointer=createData(lEn+defaultBufLen);
		pointer->rawLen=lEn;
		for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
	}
	String(float par1):
		length(pointer)
	{
		char tmp[50];
		sprintf(tmp,floatFormat,par1);
		removeZeros(tmp);
		unsigned lEn=strlen(tmp);
		pointer=createData(lEn+defaultBufLen);
		pointer->rawLen=lEn;
		for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
	}
	String(double par1):
		length(pointer)
	{
		char tmp[50];
		sprintf(tmp,floatFormat,par1);
		removeZeros(tmp);
		unsigned lEn=strlen(tmp);
		pointer=createData(lEn+defaultBufLen);
		pointer->rawLen=lEn;
		for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
	}

	~String(){
		if(--pointer->refCount==0){
			delete pointer;
		}
		pointer=NULL;
	}
	String& operator= (const String& par1){
		par1.pointer->refCount++;
		if(--pointer->refCount==0){
			delete pointer;
		}
		pointer=par1.pointer;
		return *this;
	}
	String& operator= (const char* par1){
		unsigned lEn=strlen(par1);

		if(pointer->refCount>1||lEn>pointer->bufLen){
			if(--pointer->refCount==0){
				delete pointer;
			}
			pointer=createData(lEn+defaultBufLen);
		}
		pointer->rawLen=lEn;
		for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=par1[i];
		return *this;
	}
	friend String	operator + (const String& par1, const String& par2){
		return String(par1)+=par2;
	}
	String& operator= (const char& par);
	String& operator= (float par1);
	String& operator= (int par1);


    const char *showReason() const { 
		return "Exception in String class."; 
	}
	const char& operator ()(const int par1)const{
		if((unsigned)par1>=(unsigned)pointer->rawLen){
			throw this;
		}
		return pointer->rawChar[par1];
	}
	/*
	operator const String ()const{
		return *this;
	}
	*/
	const char* c_str()const{
		return pointer->rawChar;
	}
	int getBufLength()const{
		return pointer->bufLen;
	}




////////////////////////////
    friend bool	operator !=(const String &par1, const String &par2){
		return (strcmp(par1.pointer->rawChar, par2.pointer->rawChar))!=0;
	}

    friend bool	operator !=(const char *par1, const String &par2){
		return (strcmp(par1, par2.pointer->rawChar))!=0;
	}

    friend bool	operator !=(const String& par1, const char* par2){
		return (strcmp(par1.pointer->rawChar, par2))!=0;
	}

    friend bool	operator ==(const String &par1, const String &par2){
		return (!strcmp(par1.pointer->rawChar, par2.pointer->rawChar))!=0;
	}

    friend bool	operator ==(const char *par1, const String &par2){
		return (!strcmp(par1, par2.pointer->rawChar))!=0;
	}
    
    friend bool	operator ==(const String& par1, const char* par2){
		return (!strcmp(par1.pointer->rawChar, par2))!=0;
	}
	void requestMoreIfNeeded(unsigned more){
		unsigned i;
		if(pointer->refCount>1||pointer->rawLen+more>pointer->bufLen){
			Data* newPointer=createData(pointer->rawLen+more+defaultBufLen, pointer->rawLen);
			for(i=0;i<pointer->rawLen+1;i++){
				newPointer->rawChar[i]=pointer->rawChar[i];
			}
			if(--pointer->refCount==0){
				delete pointer;
			}
			pointer=newPointer;
		}
	}
	String &operator +=(const String &par1);
	String operator +=(char par1);
	String operator +=(int par1){
		requestMoreIfNeeded(20);
		char* p=pointer->rawChar+pointer->rawLen;
		sprintf(p,"%i",par1);
		pointer->rawLen+=strlen(p);
		return *this;
	}
	String operator +=(float par1){
		requestMoreIfNeeded(20);
		char* p=pointer->rawChar+pointer->rawLen;
		sprintf(p,floatFormat,par1);
		removeZeros(p);
		pointer->rawLen+=strlen(p);
		return *this;
	}
	String operator +=(double par1){
		requestMoreIfNeeded(20);
		char* p=pointer->rawChar+pointer->rawLen;
		sprintf(p,floatFormat,par1);
		removeZeros(p);
		pointer->rawLen+=strlen(p);
		return *this;
	}
	String operator +=(const char* par1){
		int length=strlen(par1);
		requestMoreIfNeeded(length);
		for(int i=0;i<length+1;i++){
			pointer->rawChar[pointer->rawLen+i]=par1[i];
		}
		pointer->rawLen+=length;
		return *this;
	}
	void insert(const char& ch, const int& index){
		(*this)+=' ';
		unsigned int i=pointer->rawLen;
		for(; --i>(unsigned)index;){
			(*this)[i]=(*this)[i-1];
		}
		(*this)[i]=ch;
	}
	void deleteChar(const int& index){
		*this=replace(index, 1, "");
	}
	const String substr(const int& index, const int desiredLen=0x7fffffff) const {
		unsigned i;
		unsigned realLen=length-(const unsigned)index; //potential length stage
		if((int)realLen<0)throw this;
		if(realLen>(const unsigned)desiredLen){
			realLen=desiredLen;
		}
		Data* newPointer=createData(realLen+defaultBufLen);
		newPointer->rawLen=realLen;
		for(i=0;i<realLen;i++){
			newPointer->rawChar[i]=pointer->rawChar[index+i];
		}
		newPointer->rawChar[i]=0;
		return newPointer;
	}
	const String substring(const int& startChar, const int endChar=-1) const {
		unsigned strLength,i;
		if(endChar==-1){
			strLength=pointer->rawLen-startChar;
		}
		else{
			strLength=endChar-startChar+1;
		}
		if((unsigned)startChar+strLength>pointer->rawLen)throw this;

		Data* newPointer=createData(strLength+defaultBufLen);
		newPointer->rawLen=strLength;
		for(i=0;i<strLength;i++){
			newPointer->rawChar[i]=pointer->rawChar[startChar+i];
		}
		newPointer->rawChar[i]=0;
		return newPointer;
	}
	String replace(const int& index, const int& desiredLen, const String strPar) const{
		String str=substr(0,index);
		str+=strPar;
		unsigned newIndex=index+desiredLen;
		if(newIndex<(unsigned)length){
			str+=substr(newIndex);
		}
		return str;
	}
	char& operator [](const int& par1){
		if((unsigned)par1>=(unsigned)pointer->rawLen){
			throw this;
		}
		if(pointer->refCount>1){
			Data* newPointer=createData(pointer->bufLen, pointer->rawLen);
			unsigned i=0;
			do{
				newPointer->rawChar[i]=pointer->rawChar[i];
			}while(i++<pointer->rawLen);

			pointer=newPointer;
		}
		return pointer->rawChar[par1];
	}
	const char& operator [](const int& par1)const{
		if((unsigned)par1>=(unsigned)pointer->rawLen){
			throw this;
		}
		return pointer->rawChar[par1];
	}
/*
	int fromText(char*& p,int i=0);
	int readWord(char* p){
		for(length=0;p[length]>' ';++length){
		}
		pointer=new SFChar[length+1];
		for(int i=0;i<length;i++,p++){
			pointer[i]=*p;
		}
		pointer[i]=0;
		return length;
	}
	*/
	int indexOf (String searchString, int position=0)const;

	void getField(const String& str, int& i);
	String upperCase()const{
		String str=*this;
		char* p=str.pointer->rawChar;
		for(unsigned i=0; i<pointer->rawLen; i++){
			if(p[i]>='a'&&p[i]<='z'){
				str[i]-=0x20;
			}
		}
		return str;
	}
	String lowerCase()const{
		String str=*this;
		char* p=str.pointer->rawChar;
		for(unsigned i=0; i<pointer->rawLen; i++){
			if(p[i]>='A'&&p[i]<='Z'){
				str[i]+=0x20;
			}
		}
		return str;
	}
	
String toString();
void endOfName(int& index)const;
void endOfCppLabel(int& index)const;
String extractName(int& i)const;
String extractCppLabel(int& i)const;
//String extractCppOperator(int& i)const;

void stb(String& dest, int& position)const;
void stw(String& dest, int& position)const;

void stb(int& position)const; //searchTillBlack treats coma as white
void stbCpp(int& position)const; //searchTillBlack includes coma as black
void stbnlCpp(int& position)const;//searchTillBlack or new line includes coma as black
void stbc(int& position)const;//searchTillBlackComment
void stw(int& position)const; //searchTillWhite
void stwo(int& position)const; //searchTillWhiteOrOperator //try not to use this
void stwo2(int& position)const; //searchTillWhiteOrOperator //use this
void stw_stb(int& position)const; //searchTillWhite then searchTillBlack
void stwo_stb(int& position)const; //searchTillWhite or Operator then search till black
void stnl(int& position)const;//searchTill new line
//void stNonCppOperator(int& position)const; //searchTillNonOperator (C++)
int getLengthOfCppOperator(int position)const;
void endOfNum(int& position)const;


bool cmpKWord(const char* str,int& index)const;
bool cmpKWord(const String& str,int& index)const;
bool cmpCppWord(const char* str,int& index)const;
bool cmpCppWord(const String& str,int& index)const;
bool findWord( const String str, int& i)const;
String fromQuotes( int& position)const;
String getWord(int& position)const;
String getWord2(int& position)const;

	void write(FILE* file)const{
		fwrite(pointer->rawChar,1,pointer->rawLen,file);
	}
	void write(FILE* stream, char* name)const;
	bool write(String fileName)const;
	//bool read( String fileName);
//private:

	Length length;
private:
	Data* pointer;
};

template <typename T > class M_;

template <class S> void iinsertElement(M_<S>& t, int targetIndex, S& value){
	int i=t.length;
	t.length++;
	if(t.length>1){
		do{
			--i;
			t[i+1]=t[i];
		}while(i>targetIndex);

	}
	t[targetIndex]=value;
};

template <typename T>class M_ {
class Length{
	friend class M_;
protected:
	unsigned bufLen;
	unsigned rawLen;
	T* pointer;
public:
	Length():
		rawLen(0), bufLen(0), pointer(0)
	{
	}
	Length(const T& f0):
		rawLen(1),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
	}
	Length(const T& f0,const T& f1):
		rawLen(2),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
	}
	Length(const T& f0,const T& f1,const T& f2):
		rawLen(3),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3):
		rawLen(4),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4):
		rawLen(5),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5):
		rawLen(6),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6):
		rawLen(7),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7):
		rawLen(8),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8):
		rawLen(9),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9):
		rawLen(10),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10):
		rawLen(11),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11):
		rawLen(12),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
		pointer[11]=f11;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12):
		rawLen(13),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
		pointer[11]=f11;
		pointer[12]=f12;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13):
		rawLen(14),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
		pointer[11]=f11;
		pointer[12]=f12;
		pointer[13]=f13;
	}
	Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13, const T& f14, const T& f15):
		rawLen(16),
		bufLen(STANDARDBUFLEN)
	{
		pointer=new T[STANDARDBUFLEN];
		pointer[0]=f0;
		pointer[1]=f1;
		pointer[2]=f2;
		pointer[3]=f3;
		pointer[4]=f4;
		pointer[5]=f5;
		pointer[6]=f6;
		pointer[7]=f7;
		pointer[8]=f8;
		pointer[9]=f9;
		pointer[10]=f10;
		pointer[11]=f11;
		pointer[12]=f12;
		pointer[13]=f13;
		pointer[14]=f14;
		pointer[15]=f15;
	}
	Length(const M_& param):
		rawLen(param.length.rawLen),
		bufLen(param.length.bufLen),
		pointer(new T[bufLen])
	{
		for(unsigned i=0;i<rawLen;i++)pointer[i]=param.length.pointer[i];
	}
	~Length(){
		delete[] pointer;
	}
	int operator =(const Length& par){
		unsigned newLength=par.rawLen;
		unsigned i;
		if(bufLen<newLength){
			bufLen=newLength*2;
			T* newPointer=new T[bufLen];
			for(i=0;i<rawLen;i++){
				newPointer[i]=pointer[i];
			}
			delete[] pointer;
			pointer=newPointer;
		}
		for(i=rawLen;i<newLength;i++){
			pointer[i]=T();
		}
		rawLen=newLength;
		return rawLen;
	}
	inline int operator =(const int& newLength){
		unsigned i;
		if(bufLen<(unsigned)newLength){
			bufLen=newLength*2;
			T* newPointer=new T[bufLen];
			for(i=0;i<rawLen;i++){
				newPointer[i]=pointer[i];
			}
			delete[] pointer;
			pointer=newPointer;
		}
		for(i=rawLen;i<(unsigned)newLength;i++){
			pointer[i]=T();
		}
		rawLen=newLength;
		return rawLen;
	}
	int operator ++ (int){
		unsigned newLength=rawLen+1;

		if(bufLen<newLength){
			bufLen=newLength*2;
			T* newPointer=new T[bufLen];
			for(unsigned i=0;i<rawLen;i++){
				newPointer[i]=pointer[i];
			}
			delete[] pointer;
			pointer=newPointer;
		}
		pointer[rawLen]=T();
		rawLen=newLength;
		return rawLen-1;
	}
	int operator -- (int){
		rawLen--;
		if((int)rawLen<0)throw this;
		return rawLen;
	}
	int operator -= (int par){
		rawLen-=par;
		if((int)rawLen<0)throw this;
		return rawLen;
	}
	int operator += (int par){
		unsigned newLength=par+rawLen;

		if(bufLen<newLength){
			bufLen=newLength*2;
			T* newPointer=new T[bufLen];
			for(unsigned i=0;i<rawLen;i++){
				newPointer[i]=pointer[i];
			}
			delete[] pointer;
			pointer=newPointer;
		}
		for(unsigned i=rawLen;i<newLength;i++){
			pointer[i]=T();
		}
		rawLen=newLength;
		return newLength;
	}
	operator int ()const{
		return rawLen;
	}
};

public:
	M_():
		length()
	{}
	M_(const T& f0):
		length(f0)
	{}
	M_(const T& f0,const T& f1):
		length(f0,f1)
	{}
	M_(const T& f0,const T& f1,const T& f2):
		length(f0,f1,f2)
	{}
	M_(const T& f0,const T& f1,const T& f2,const T& f3):
		length(f0,f1,f2,f3)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4):
		length(f0,f1,f2,f3,f4)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5):
		length(f0,f1,f2,f3,f4,f5)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6):
		length(f0,f1,f2,f3,f4,f5,f6)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7):
		length(f0,f1,f2,f3,f4,f5,f6,f7)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13)
	{}
	M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13, const T& f14, const T& f15):
		length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13,f14,f15)
	{}
	M_(const M_& param):
		length(param)
	{}
	~M_(){
	}
	M_& operator= (const M_& par){
		if(par.length.pointer==0){
			length.rawLen=0;
		}
		else if(par.length.pointer!=length.pointer){
			if(length.bufLen<par.length.bufLen){
				delete[] length.pointer;
				length.rawLen=par.length.rawLen;
				length.bufLen=length.rawLen*2;
				length.pointer=new T[length.bufLen];
			}
			else{
				length.rawLen=par.length.rawLen;
			}
			for(unsigned i=0;i<length.rawLen;i++)length.pointer[i]=par.length.pointer[i];
		}
		return *this;
	}
	//const M_& operator =(const SF<M_,	T>& par);

	M_& operator +=(T par){
		unsigned newLength=length.rawLen+1;

		if(length.bufLen<newLength){
			length.bufLen=newLength*2;
			T* newPointer=new T[length.bufLen];
			for(unsigned i=0;i<length.rawLen;i++){
				newPointer[i]=length.pointer[i];
			}
			delete[] length.pointer;
			length.pointer=newPointer;
		}
		length.pointer[length.rawLen]=par;
		length.rawLen=newLength;
		return *this;
	}
	M_& operator +=(const M_& par1){
		unsigned i;
		unsigned newLength=par1.length.rawLen+length.rawLen;
		if(length.bufLen<newLength){
			length.bufLen=newLength*2;
			T* newPointer=new T[length.bufLen];
			for(unsigned i=0;i<length.rawLen;i++){
				newPointer[i]=length.pointer[i];
			}
			delete[] length.pointer;
			length.pointer=newPointer;
		}
		for(i=0;i<par1.length.rawLen;i++){
			length.pointer[i+length.rawLen]=par1.length.pointer[i];
		}
		length.rawLen=newLength;
		return *this;
	}
	M_ operator +(const T& par)const{
		M_ m=*this;
		m+=par;
		return m;
	}
	M_ operator +(const M_& par)const{
		M_ m=*this;
		m+=par;
		return m;
	}
    const char *showReason() const { 
		return "Exception in M_ class."; 
	}
	int size()const{
		return length.rawLen;
	}
	void setElement(const int index, const T& val){
		length.pointer[index]=val;
	}
	T& operator [](const int& index){
		if((unsigned)index>=length.rawLen){
			throw this;
			/*
			unsigned newLength=index+1;
			unsigned i;
			if(length.bufLen<newLength){
				length.bufLen=newLength*2;
				T* newPointer=new T[length.bufLen];
				for(i=0;i<length.rawLen;i++){
					newPointer[i]=length.pointer[i];
				}
				delete[] length.pointer;
				length.pointer=newPointer;
			}
			for(i=length.rawLen;i<newLength;i++){
				length.pointer[i]=T();
			}
			length.rawLen=newLength;
			*/
		}
		return length.pointer[index];
	}
	const T& operator [](const int& index)const{
		if((unsigned)index>=length.rawLen){
			throw this;
		}
		return length.pointer[index];
	}
	const T* getArray()const{
		return length.pointer;
	}
	T* getArray(){
		return length.pointer;
	}
	bool operator ==(const M_& par)const{
		if(M_::rawLen!=par->rawLen){
			return false;
		}
		for(int i=length; --i>=0;){
			if(length.pointer[i]!=par[i]){
				return false;
			}
		}
		return true;
	}
	bool operator !=(const M_& par)const{
		if(length!=par.length){
			return true;
		}
		for(int i=length; --i>=0;){
			if(length.pointer[i]!=par[i]){
				return true;
			}
		}
		return false;
	}
	void getField(const String& str,int& i);

	//'class M_<int> &(class SF<class M_<int>,int,69> &)'
//inline    M_<T, type> &	operator =(SF <M_<T,type>,T,type>& par);
//inline    M_<T, type> &	operator +=(const T par);
	void toMNodeString(String& str, bool newList=true);
	void removeNode(T par1);
	void removeNode(M_& par1);
	int& refIndex(int i)const;
	int& refTexIndex(int i)const;
	int getTriangleLength()const{
		return length*3;
	};
	bool write(String fileName)const;

	void rotate(Rotation rot,int offset=0);
	void rotate( float x, float y, float z, float angle, int offset=0);
	void translate(T tran,int offset=0);
	void translate( float x, float y, float z, int offset=0);
	void scale(float xsc,float ysc,float zsc);
	void translateToZero(int offset,int numOf, Vec3f& center);
	void insertElement(int targetIndex, T value){
		iinsertElement(*this, targetIndex, value);
	}

	void deleteElement(int targetIndex);
	void deleteElement(int from, int numOf);

	void toCppString(String& str);


	void translateToZero(int offset,int numOf,Rotation& center);
	void setUpRoute( int& yPos);
	int getIndex(String str)const;
	Length length;
};
template <class T, class L> void removeTypeFromList(M_<Ptr<L> >& list){
	int d=0;
	int s=0;
	while(s<list.length){
		if(dynamic_cast<T*>(list[s].p)){
			list[s]=0;
			s++;
			continue;
		}
		list[d++]=list[s++];
	}
	list.length=d;
};

int hexStrToInt(const String& str, int& i);

template <class T>class Ptr {
public:
	Ptr(){
		p=0;
	}
Ptr( T* par){
	p=par;
	if(par){
		par->incRefCount();
	}
}
Ptr(const Ptr& par){
	p=par.p;
	if(p!=NULL){
		p->incRefCount();
	}
}
Ptr& operator = (const Ptr& par){
	par.p->incRefCount();
	if(p!=NULL){
		if(p->decRefCount()==0){
			delete p;
		}
	}
	p=par.p;
	return *this;
}
Ptr& operator = (int par){
	if(par!=0){
		throw this;
	}
	if(p!=NULL){
		if(p->decRefCount()==0){
			delete p;
		}
	}
	p=0;
	return *this;
}
Ptr& operator = ( T* par){
	par->incRefCount();
	if(p!=NULL){
		if(p->decRefCount()==0){
			delete p;
		}
	}
	p=par;
	return *this;
}
~Ptr(){
	if(p!=NULL&&p->decRefCount()==0){
		delete p;
		p=NULL;
	}
}
	bool	operator ==(const Node*& par)const{
		return p==par;
	}
	bool	operator !=(const Node*& par)const{
		return p!=par;
	}
	
	operator T*()const{
		return p;
	}

	T& operator *(){
		return *p;
	}
	T* operator ->()const{
		return p;
	}
	T* getRawPtr()const{
		return p;
	}
//protected:
	T* p;
};
int getInt(const String& str, int& i);

template <class T> void M_<T>::deleteElement(int i){
	for(; i < length-1; i++){
		(*this)[i]=(*this)[i+1];
	}
	length--;
}
